{--------------------------------------------------------------------------------
* Description : TscScrollingText
  Simple component to scroll text lines from bottom to top or
  top to bottom. Text lines can be added while text is scrolling.
  This component is derived from a TGraphicControl.
* Dates : December 2001 - April 2002
* Version : 1.2

* Author : Stefan Cruysberghs
* Email : stefancr@scip.be
* Website : http://www.scip.be
  Visit SCIP.be for other components, tools, articles, ...
--------------------------------------------------------------------------------
* $Archive: /Component Library/SC_Public/scScrollingText.pas $
* $Author: Stefancr $
* $Date: 19/06/03 15:59 $
* $Modtime: 27/04/02 13:42 $
* $Revision: 1 $
--------------------------------------------------------------------------------
* This component is free of charge.
* The author doesn't give a warranty for error free running
  of this component and he doesn't give any support.
* Suggestions and bugs can be send by email.
--------------------------------------------------------------------------------
Installation
* Install the TscScrollingText component by adding the unit scScrollingText.pas
  to a package. Remove the DCR file in the package. Compile and Install the package.
* The TscScrollingText component can be found in the tabsheet 'SC'
  of the component palette.
--------------------------------------------------------------------------------
History
Version 1.2 (27/4/2002)
- Some improvements
Version 1.1 (2/1/2002)
- Added MovePixels and Interval property
Version 1.0 (15/12/2001)
- First version
--------------------------------------------------------------------------------
Example 1

  scScrollingText1.ClearTextLines;
  scScrollingText1.AddTextLine('Scrolling text');
  scScrollingText1.AddTextLine('TscScrollingText component');
  scScrollingText1.AddTextLines(MemoLines.Lines);
  scScrollingText1.ScrollDirection := sdUp;
  scScrollingText1.Start;

--------------------------------------------------------------------------------}

unit scScrollingText;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls;

type
  TscScrollDirection = (sdDown, sdUp);
  TscScrollingMode = (smOff, smPause, smOn);

  TscScrollingText = class(TGraphicControl)
  private
    FTimer : TTimer;
    FStrings : TStrings;
    FIntLeftMargin : Integer;
    FIntInterval : Integer;
    FIntMovePixels : Integer;
    FScrollDirection : TscScrollDirection;
    FScrollingMode : TscScrollingMode;
    FIntLineHeight : Integer;
    FIntTextHeight : Integer;
    FIntCountTextLines : Integer;
    FIntPosY : Integer;
    FBitmap : TBitmap;
  protected
    procedure DoTimer(Sender: TObject);
  public
    constructor Create(AOwner : Tcomponent); override;
    destructor Destroy; override;

    // Start scrolling
    procedure Start; virtual;
    // Pause scrolling, text stops
    procedure Pause; virtual;
    // Stop scrolling and clear paintbox
    procedure Stop; virtual;

    // Add a text line (before or while scrolling)
    procedure AddTextLine(const StrText : String); virtual;
    // Add more text lines (before or while scrolling)
    procedure AddTextLines(SLText: TStrings); virtual;
    // Clear all text lines
    procedure ClearTextLines; virtual;
  published
    // Left margin of text
    property LeftMargin : Integer read FIntLeftMargin write FIntLeftMargin default 20;
    // Interval of internal timer
    // Change Interval and MovePixels to make it faster or slower
    property Interval : Integer read FIntInterval write FIntInterval default 50;
    // Number of pixels to add when scrolling, best result 1 pixel
    property MovePixels : Integer read FIntMovePixels write FIntMovePixels default 1;
    // Up = bottom to top, Down = top to bottom
    property ScrollDirection : TscScrollDirection read FScrollDirection write FScrollDirection default sdUp;
    // Height of text line
    property LineHeight : Integer read FIntLineHeight write FIntLineHeight default 20;

    // Read only property, current scrolling mode
    // Depends on use of methods Start, Pause, Stop
    property ScrollingMode : TscScrollingMode read FScrollingMode;

    property Color;
    property Font;
    property Anchors;
    property Left;
    property Top;
    property Width;
    property Height;
  end;

procedure Register;

implementation

{$R scScrollingText.dcr}

procedure Register;
begin
  RegisterComponents('SC', [TscScrollingText]);
end;

//------------------------------------------------------------------------------
constructor TscScrollingText.Create(AOwner: Tcomponent);
begin
  inherited;
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.OnTimer := DoTimer;
  FStrings := TStringList.Create;
  FBitmap := TBitmap.Create;

  FIntLeftMargin := 20;
  FIntInterval := 50;
  FIntMovePixels := 1;
  FScrollDirection := sdUp;
  FIntLineHeight := 20;
  FScrollingMode := smOff;
end;

//------------------------------------------------------------------------------
destructor TscScrollingText.Destroy;
begin
  FTimer.Enabled := False;
  FTimer.Free;
  FStrings.Free;
  FBitmap.Free;
  inherited;
end;

//------------------------------------------------------------------------------
procedure TscScrollingText.DoTimer(Sender: TObject);
var
  i : Integer;
begin
  FIntTextHeight := (FStrings.Count * FIntLineHeight);

  with FBitmap.Canvas do
  begin
    FillRect(Rect(0,0,Width,Height));

    for i:=0 to FStrings.Count-1 do
    begin
      TextOut(FIntLeftMargin,FIntPosY,FStrings[i]);

      if FScrollDirection = sdUp then
        FIntPosY := FIntPosY + FIntLineHeight
      else
        FIntPosY := FIntPosY - FIntLineHeight;
    end;

    if FScrollDirection = sdUp then
      FIntPosY := FIntPosY - FIntMovePixels - FIntTextHeight
    else
      FIntPosY := FIntPosY + FIntMovePixels + FIntTextHeight;

    if FStrings.Count > 0 then
    begin
      if FScrollDirection = sdUp then
      begin
        if FIntPosY < 0 then
        begin
          FStrings.Delete(0);
          FIntPosY := FIntPosY + FIntLineHeight;
          for i:= 0 to (FIntCountTextLines - FStrings.Count) do
            FStrings.Add('');
        end;
      end
      else
      begin
        if FIntPosY > Height then
        begin
          FStrings.Delete(0);
          FIntPosY := FIntPosY - FIntLineHeight;
          for i:= 0 to (FIntCountTextLines - FStrings.Count) do
            FStrings.Add('');
        end;
      end;
    end;
  end;

  Canvas.Draw(0,0,FBitmap);
end;

//------------------------------------------------------------------------------
procedure TscScrollingText.Start;
var
  i : Integer;
begin
  if FScrollingMode = smOff then
  begin
    FBitmap.Width := Width;
    FBitmap.Height := Height;
    FBitmap.Canvas.Font.Assign(Font);
    FBitmap.Canvas.Brush.Color := Color;

    FIntCountTextLines := (Height div FIntLineHeight);
    for i:= 0 to FIntCountTextLines + 1 do
      FStrings.Insert(0,'');

    FTimer.Interval := FIntInterval;
    FTimer.Enabled := True;
    FScrollingMode := smOn;

    if FScrollDirection = sdUp then
      FIntPosY := -FIntLineHeight
    else
      FIntPosY := Height;
  end
  else
    if FScrollingMode = smPause  then
    begin
      FTimer.Enabled := True;
      FScrollingMode := smOn;
    end;
end;

//------------------------------------------------------------------------------
procedure TscScrollingText.Pause;
begin
  if FScrollingMode = smOn  then
  begin
    FTimer.Enabled := False;
    FScrollingMode := smPause;
  end
  else
    if FScrollingMode = smPause then
    begin
      FTimer.Enabled := True;
      FScrollingMode := smOn;
    end;
end;

//------------------------------------------------------------------------------
procedure TscScrollingText.Stop;
begin
  FTimer.Enabled := False;
  FScrollingMode := smOff;
  FBitmap.Canvas.FillRect(Rect(0,0,Width,Height));
  Canvas.Draw(0,0,FBitmap);
end;

//------------------------------------------------------------------------------
procedure TscScrollingText.AddTextLine(const StrText: String);
begin
  FStrings.Add(StrText);
end;

//------------------------------------------------------------------------------
procedure TscScrollingText.AddTextLines(SLText: TStrings);
begin
  FStrings.AddStrings(SLText);
end;

//------------------------------------------------------------------------------
procedure TscScrollingText.ClearTextLines;
begin
  FStrings.Clear;
end;

end.
